@@ -1,3 +1,5 @@
+# I guess there is a big bug in the scales of the components --> hack this
+
 # faire un draw_at_scale and merge draw fill etc by just putting an option --> much simpler and that way you can keep original size which is the best idea
 # set to width doit juste calculer le scaling factor in fact.... --> TODO do that and same for all objects --> no cloning and all will be simpler
 
@@ -19,14 +21,27 @@
 # TODO Add the crops --> see how though and see how to warn when width or height reaches 0...
 # should I offer max projections too ??? maybe or not --> see how to do that...
 
+# the only advantage I see for rect2D vs qrectf is that they allow for rotation ????
+# see how to best handle that ???
+
+import numpy as np
+import matplotlib.pyplot as plt
+from PyQt5 import QtGui
+from PyQt5.QtGui import QPainter, QColor
+from PyQt5.QtSvg import QSvgGenerator
+import os
+from epyseg.draw.shapes.line2d import Line2D
 from epyseg.draw.shapes.point2d import Point2D
 from epyseg.figure.alignment import alignRight, alignLeft, alignTop, alignBottom, alignCenterH, alignCenterV, packY, \
     packX, packYreverse
 from epyseg.draw.shapes.rect2d import Rect2D
 from epyseg.draw.shapes.scalebar import ScaleBar
 from epyseg.draw.shapes.txt2d import TAText2D
-from epyseg.img import Img
-from PyQt5.QtCore import QRectF, QPointF
+from epyseg.figure.fig_tools import preview
+# from epyseg.figure import fig_tools
+from epyseg.img import Img, toQimage
+from PyQt5.QtCore import QRectF, QPointF, QSize, QRect
+# from sympy import nsolve, exp, Symbol
 # logger
 from epyseg.tools.logger import TA_logger
 
@@ -71,12 +86,15 @@ class Image2D(Rect2D):
                 elif isinstance(args[0], Img):
                     self.filename = None
                     self.img = args[0]
-                    self.qimage = self.img.getQimage()
+                    self.qimage =  toQimage(self.img)
                     if x is None:
                         x = 0
                     if y is None:
                         y = 0
-                    super(Image2D, self).__init__(x, y, self.img.get_width(), self.img.get_height())
+                    try:
+                        super(Image2D, self).__init__(x, y, self.img.get_width(), self.img.get_height())
+                    except:
+                        super(Image2D, self).__init__(x, y, self.img.shape[1], self.img.shape[0])
                     self.isSet = True
         else:
             self.filename = None
@@ -91,7 +109,7 @@ class Image2D(Rect2D):
             except:
                 logger.error('could not load image '+str(self.filename))
                 return
-            self.qimage = self.img.getQimage()
+            self.qimage = toQimage(self.img)
             width = self.img.get_width()
             height = self.img.get_height()
             super(Image2D, self).__init__(0, 0, width, height)
@@ -103,7 +121,7 @@ class Image2D(Rect2D):
         elif data is None:
             if self.filename is not None:
                 self.img = Img(self.filename)
-                self.qimage = self.img.getQimage()
+                self.qimage =  toQimage(self.img)
                 if x is None:
                     x = 0
                 if y is None:
@@ -113,7 +131,7 @@ class Image2D(Rect2D):
         elif data is not None:
             self.img = Img(data,
                            dimensions=dimensions)  # need width and height so cannot really be only a numpy stuff --> cause no width or height by default --> or need tags such as image type for dimensions
-            self.qimage = self.img.getQimage()
+            self.qimage =  toQimage(self.img)
             # need Image dimensions id data is not of type IMG --> could check that
             if x is None:
                 x = 0
@@ -131,9 +149,15 @@ class Image2D(Rect2D):
     # @return the block incompressible height
     def getIncompressibleHeight(self):
         extra_space = 0  # can add some if boxes around to add text
+        # do not even do that --> just ignore things
+        # add box around ???
         return extra_space
 
     def add_object(self, object, position):
+        if isinstance(object, list):
+            for obj in object:
+                self.add_object(obj, position=position)
+            return
         if position == Image2D.TOP_LEFT:
             self.top_left_objects.append(object)
         elif position == Image2D.BOTTOM_RIGHT:
@@ -219,8 +243,13 @@ class Image2D(Rect2D):
             if self.img is not None:
                 x = 0
                 y = 0
-                w = self.img.get_width()
-                h = self.img.get_height()
+                try:
+                    w = self.img.get_width()
+                    h = self.img.get_height()
+                except:
+                    w = self.img.shape[1]
+                    h = self.img.shape[0]
+
                 if self.__crop_top is not None:
                     y = self.__crop_top
                     h -= self.__crop_top
@@ -570,6 +599,7 @@ class Image2D(Rect2D):
         scale = width_in_px / pure_image_width
         self.scale = scale
 
+
     def setToHeight(self, height_in_px):
         # pure_image_height = self.height()
         # self.setHeight(height_in_px)
@@ -584,6 +614,8 @@ class Image2D(Rect2D):
         #     pure_image_height-=self.__crop_bottom
         scale = height_in_px/pure_image_height
         self.scale = scale
+        # need update bounds
+        # scale is ok
 
     # def update_scale(self):
     #     # we recompute the scale for the scale bar # TODO BE CAREFUL IF EXTRAS ARE ADDED TO THE OBJECT AS THIS WOULD PERTURB THE COMPUTATIONS
@@ -626,6 +658,7 @@ class Image2D(Rect2D):
     def set_to_translation(self, translation):
         self.translation = translation
 
+    # WHY 0,0 FOR BOUNDS ??? --> sounds not smart to me
     def boundingRect(self, scaled=True):
  # en fait pas good besoin de prendre les crops et le scale en compte
         # is
@@ -676,6 +709,148 @@ class Image2D(Rect2D):
     def height(self, scaled=True):
         return self.boundingRect(scaled=scaled).height()
 
+    SVG_INKSCAPE = 96
+    SVG_ILLUSTRATOR = 72
+
+    # NB THIS CODE IS BASED ON THE EZFIG SAVE CODE --> ANY CHANGE MADE HERE MAY ALSO BE MADE TO THE OTHER
+    # best here would be for image to keep original size if nothing is specified
+    # could also make it return a qimage for display in pyTA
+    qualities = [QPainter.NonCosmeticDefaultPen,  QPainter.SmoothPixmapTransform, QPainter.TextAntialiasing,QPainter.Antialiasing,QPainter.HighQualityAntialiasing]
+
+    def save(self, path, filetype=None, title=None, description=None, svg_dpi=SVG_INKSCAPE, quality=qualities[-1]):
+        # if path is None or not isinstance(path, str):
+        #     logger.error('please provide a valide path to save the image "' + str(path) + '"')
+        #     return
+        if path is None:
+            filetype = '.tif'
+
+        if filetype is None:
+            if path.lower().endswith('.svg'):
+                filetype = 'svg'
+            else:
+                filetype = os.path.splitext(path)[1]
+        dpi = 72  # 300 # inkscape 96 ? check for illustrator --> check
+
+        if filetype == 'svg':
+            generator = QSvgGenerator()
+            generator.setFileName(path)
+            if svg_dpi == self.SVG_ILLUSTRATOR:
+                generator.setSize(QSize(595, 842))
+                generator.setViewBox(QRect(0, 0, 595, 842))
+            else:
+                generator.setSize(QSize(794, 1123))
+                generator.setViewBox(QRect(0, 0, 794, 1123))
+
+            if title is not None and isinstance(title, str):
+                generator.setTitle(title)
+            if description is not None and isinstance(description, str):
+                generator.setDescription(description)
+            generator.setResolution(
+                svg_dpi)  # fixes issues in inkscape of pt size --> 72 pr illustrator and 96 pr inkscape but need change size
+
+            painter = QPainter(generator)
+
+            # print(generator.title(), generator.heightMM(), generator.height(), generator.widthMM(),
+            #       generator.resolution(), generator.description(), generator.logicalDpiX())
+        else:
+            scaling_factor_dpi = 1
+            # scaling_factor_dpi = self.scaling_factor_to_achieve_DPI(300)
+
+            # in fact take actual page size ??? multiplied by factor
+            # just take real image size instead
+
+
+            # image = QtGui.QImage(QSize(self.cm_to_inch(21) * dpi * scaling_factor_dpi, self.cm_to_inch(29.7) * dpi * scaling_factor_dpi), QtGui.QImage.Format_RGBA8888) # minor change to support alpha # QtGui.QImage.Format_RGB32)
+
+            # NB THE FOLLOWING LINES CREATE A WEIRD ERROR WITH WEIRD PIXELS DRAWN some sort of lines NO CLUE WHY
+
+            img_bounds = self.boundingRect()
+            image = QtGui.QImage(QSize(img_bounds.width() * scaling_factor_dpi, img_bounds.height()* scaling_factor_dpi),  QtGui.QImage.Format_RGBA8888)  # minor change to support alpha # QtGui.QImage.Format_RGB32)
+            # print('size at dpi',QSize(img_bounds.width() * scaling_factor_dpi, img_bounds.height()* scaling_factor_dpi))
+            # QSize(self.cm_to_inch(0.02646 * img_bounds.width())
+            # self.cm_to_inch(0.02646 * img_bounds.height())
+            # need convert pixels to inches
+            # is there a rounding error
+
+            # force white bg for non jpg
+            try:
+                # print(filetype.lower())
+                # the tif and png file formats support alpha
+                if not filetype.lower() == '.png' and not filetype.lower() == '.tif' and not filetype.lower() == '.tiff':
+                    image.fill(QColor.fromRgbF(1,1,1))
+                else:
+                    # image.fill(QColor.fromRgbF(1, 1, 1, alpha=1))
+                    # image.fill(QColor.fromRgbF(1, 1, 1, alpha=1))
+                    # TODO KEEP in fact image need BE FILLED WITH TRANSPARENT OTHERWISE GETS WEIRD DRAWING ERRORS
+                    # TODO KEEP SEE https://stackoverflow.com/questions/13464627/qt-empty-transparent-qimage-has-noise
+                    # image.fill(qRgba(0, 0, 0, 0))
+                    image.fill(QColor.fromRgbF(0,0,0,0))
+            except:
+                pass
+            painter = QPainter(image)  # see what happens in case of rounding of pixels
+            # painter.begin()
+            painter.scale(scaling_factor_dpi, scaling_factor_dpi)
+        painter.setRenderHint(quality)  # to improve rendering quality
+        self.draw(painter)
+        painter.end()
+        if path is None:
+            return image
+        if filetype != 'svg':
+            image.save(path)
+            return image
+
+    #based on https://stackoverflow.com/questions/19902183/qimage-to-numpy-array-using-pyside
+    def convert_qimage_to_numpy(self, qimage):
+        qimage = qimage.convertToFormat(QtGui.QImage.Format.Format_RGB32)
+        width = qimage.width()
+        height = qimage.height()
+        image_pointer = qimage.bits() # creates a deep copy --> this is what I want
+        image_pointer.setsize(qimage.byteCount())
+        # arr = np.array(image_pointer,copy=True).reshape(height, width, 4)
+        arr = np.array(image_pointer).reshape(height, width, 4)
+        arr = arr[..., 0:3]
+        arr = Img.RGB_to_BGR(arr)  # that seems to do the job
+        return arr
+
+    # simple equation for an image
+    # or maybe ask for starting expression
+    # autoincrement math to get the right value
+    # def _get_width_equation(self):
+    #     # incr = fig_tools.common_value
+    #     # AR = self.width(False)/self.height(False)
+    #     # variable =
+    #     equa_w = Symbol('a'+str(id(self)))*self.width(False)+0
+    #     # equa_h = variable*AR*self.width(False)+0
+    #     # incr+=1
+    #     # fig_tools.common_value = incr
+    #     return equa_w #, equa_h
+
+    # def _get_height_equation(self):
+    #     # incr = fig_tools.common_value
+    #     AR = self.width(False)/self.height(False)
+    #     # equa_w = Symbol('a'+str(id(self)))*self.width(False)+0
+    #     equa_h = Symbol('a'+str(id(self)))*AR*self.width(False)+0
+    #     # incr+=1
+    #     # fig_tools.common_value = incr
+    #     return equa_h #, equa_h
+
+    # use that so that I can expand
+    # if unique --> do nothing if many see how to handle that???
+    # see how
+    def get_equation(self):
+        from sympy import nsolve, exp, Symbol
+        AR = self.width(False)/self.height(False)
+        equa_w = Symbol('a'+str(id(self)))*self.width(False)+0
+        equa_h = Symbol('a'+str(id(self)))*AR*self.width(False)+0
+        return [equa_w] , [equa_h], None # equa in width, equa in height, equas to solve, not useful here though
+
+    # allow to deepcopy and copy
+    def __copy__(self):
+        return self
+
+    def __deepcopy__(self, memo):
+        return self
+
 if __name__ == '__main__':
     # ça marche --> voici deux examples de shapes
     test = Image2D(x=12, y=0, width=100, height=100)  # could also be used to create empty image with
@@ -686,9 +861,62 @@ if __name__ == '__main__':
 
     # bug qd on definit une image comme param
     # test = Image2D('./../data/counter/06.png')
-    test = Image2D('/D/Sample_images/sample_images_PA/trash_test_mem/counter/01.png')
+    test = Image2D('/E/Sample_images/sample_images_PA/trash_test_mem/counter/01.png')
     print(test.boundingRect())  # --> it is ok there so why not below # not callable --> why -->
     print(test.get_P1())  # ça marche donc où est le bug
     print(test.get_P1().y())  # ça marche donc où est le bug
     # print(test.getP1().width())
     # ça marche
+
+    # try draw on the image the quivers
+    # img0.setLettering('<font color="red">A</font>')
+    # # letter
+    # img0.annotation.append(Rect2D(88, 88, 200, 200, stroke=3, color=0xFF00FF))
+    # img0.annotation.append(Ellipse2D(88, 88, 200, 200, stroke=3, color=0x00FF00))
+    # img0.annotation.append(Circle2D(33, 33, 200, stroke=3, color=0x0000FF))
+    test.annotation.append(Line2D(33, 33, 88, 88, stroke=3, color=0x0000FF))
+
+    test.annotation.append(Line2D(128, 33, 88, 88, stroke=0.65, color=0xFFFF00))
+    # img0.annotation.append(Freehand2D(10, 10, 20, 10, 20, 30, 288, 30, color=0xFFFF00, stroke=3))
+    # # img0.annotation.append(PolyLine2D(10, 10, 20, 10, 20, 30, 288, 30, color=0xFFFF00, stroke=3))
+    # img0.annotation.append(Point2D(128, 128, color=0xFFFF00, stroke=6))
+
+    # add a save to the image --> so that it exports as a raster --> TODO
+
+
+    # painter =
+    # img = test.draw() # if no painter --> create one as not to lose any data and or allow to save as vectorial
+
+
+    img = test.save('/E/Sample_images/sample_images_PA/trash_test_mem/mini_vide/analyzed/trash/test_line2D.tif')
+
+    test.save('/E/Sample_images/sample_images_PA/trash_test_mem/mini_vide/analyzed/trash/test_line2D.svg')
+
+    #trop facile --> just hack it so that it can return a single qimage # or return a numpy image that is then plotted -> should not be too hard !!! I think --> TODO
+    img = test.convert_qimage_to_numpy(img) # --> ok but I just need to swap the channels then I'll be done --> try that maybe with plenty of input images just to see
+
+
+    # shall I save
+    # try with non RGB images just to see
+
+    # img = Img.RGB_to_BGR(img)
+
+    # almost there --> just need to check that the size of the image is ok and that everything is fine
+    plt.imshow(img)
+    plt.show()
+
+
+    print('here')
+    img2 = Image2D('/E/Sample_images/sample_images_PA/trash_test_mem/counter/02.png')
+    print('here3')
+    preview(img2)
+
+    # test replace the plot of pyTA of the polarity by this one --> should be quite easy TODO I think --> TODO
+
+    # ok --> it all seems to work --> see how I can handle that
+
+
+
+    # --> all seems ok now
+    # --> put this in the advanced sql plotter
+
